data_list = [
    "10x-5gex-cdr3",
    "10x-5gex-full-length",
    "10x-vdj-bcr",
    "10x-vdj-bcr-full-length",
    "10x-vdj-tcr",
    "10x-vdj-tcr-full-length",
    "abhelix-human-bcr-cdr3",
    "abhelix-human-bcr-full-length",
    "abhelix-human-tcr-cdr3",
    "abhelix-human-tcr-full-length",
    "ampliseq-tcrb-plus-cdr3",
    "ampliseq-tcrb-plus-full-length",
    "ampliseq-tcrb-sr-cdr3",
    "bd-rhapsody-human-bcr-full-length",
    "bd-rhapsody-human-bcr-v1",
    "bd-rhapsody-human-bcr-v2",
    "bd-rhapsody-human-tcr-full-length",
    "bd-rhapsody-human-tcr-v1",
    "bd-rhapsody-human-tcr-v2",
    "bd-rhapsody-mouse-bcr-full-length",
    "bd-rhapsody-mouse-bcr-v1",
    "bd-rhapsody-mouse-bcr-v2",
    "bd-rhapsody-mouse-tcr-full-length",
    "bd-rhapsody-mouse-tcr-v1",
    "bd-rhapsody-mouse-tcr-v2",
    "biomed2-human-bcr-cdr3",
    "biomed2-human-bcr-full-length",
    "cellecta-air-human",
    "irepertoire-human-dna-igh-lr",
    "irepertoire-human-dna-igh-sr",
    "irepertoire-human-dna-trb-lr",
    "irepertoire-human-dna-trb-sr",
    "irepertoire-human-rna-bcr-lr",
    "irepertoire-human-rna-bcr-sr",
    "irepertoire-human-rna-repseq-plus",
    "irepertoire-human-rna-tcr-lr",
    "irepertoire-human-rna-tcr-sr",
    "irepertoire-mouse-rna-bcr-lr",
    "irepertoire-mouse-rna-bcr-sr",
    "irepertoire-mouse-rna-tcr-lr",
    "irepertoire-mouse-rna-tcr-sr",
    "mikelov-et-al-2021",
    "milab-human-bcr-multiplex-cdr3",
    "milab-human-bcr-multiplex-full-length",
    "milab-human-tcr-dna-multiplex-cdr3",
    "milab-human-tcr-rna-multiplex-cdr3",
    "milab-human-tcr-rna-race-cdr3",
    "milab-human-tcr-rna-race-full-length",
    "milab-mouse-tcr-rna-race-cdr3",
    "milab-mouse-tcr-rna-race-full-length",
    "nebnext-human-bcr-base",
    "nebnext-human-bcr-cdr3",
    "nebnext-human-bcr-full-length",
    "nebnext-human-tcr-base",
    "nebnext-human-tcr-cdr3",
    "nebnext-human-tcr-full-length",
    "nebnext-mouse-bcr-base",
    "nebnext-mouse-bcr-cdr3",
    "nebnext-mouse-bcr-full-length",
    "nebnext-mouse-tcr-base",
    "nebnext-mouse-tcr-cdr3",
    "nebnext-mouse-tcr-full-length",
    "oncomine-human-bcr-igh-lr-full-length",
    "oncomine-human-bcr-igh-sr-cdr3",
    "oncomine-human-bcr-ihg-lr-cdr3",
    "oncomine-human-bcr-pan-clonality-cdr3",
    "oncomine-human-igh-fr1-j",
    "oncomine-human-igh-fr2-j",
    "oncomine-human-igh-fr3-j",
    "oncomine-human-igh-leader-j",
    "oncomine-human-tcr-pan-clonality-cdr3",
    "oncomine-human-tcrb-lr-cdr3",
    "oncomine-human-tcrb-lr-full-length",
    "oncomine-human-tcrb-sr-cdr3",
    "parse-bio-vdj-3gex",
    "qiaseq-human-tcr-cdr3",
    "qiaseq-human-tcr-full-length",
    "qiaseq-mouse-tcr-cdr3",
    "qiaseq-mouse-tcr-full-length",
    "singleron-2.0.1-vdj-cdr3",
    "split-seq-vdj-3gex",
    "takara-human-bcr-cdr3",
    "takara-human-bcr-full-length",
    "takara-human-tcr-V1-cdr3",
    "takara-human-tcr-V1-full-length",
    "takara-human-tcr-V2-cdr3",
    "takara-human-tcr-V2-full-length",
    "takara-mouse-bcr-cdr3",
    "takara-mouse-bcr-full-length",
    "takara-mouse-tcr-cdr3",
    "takara-mouse-tcr-full-length",
    "vergani-et-al-2017-cdr3",
    "vergani-et-al-2017-full-length"
]
